@@ -174,6 +174,10 @@ void linux_boot( void )
     if (bi.mch_type == ATARI_MACH_AB40 &&
 	((unsigned long)linux_boot & 0xff000000))
 	ERROR( "Error: Bootstrap can't run in FastRAM on Afterburner040\n" );
+    /* For similar reasons, the bootstrap doesn't work in Centurbo2 TT-RAM */
+    if (getcookie("_CT2", NULL) != -1 &&
+	((unsigned long)linux_boot & 0xff000000))
+	ERROR( "Error: Bootstrap can't run in TT-RAM on Centurbo2\n" );
     
     get_mem_infos();
     /* extract data of first chunk, there goes the kernel to */
@@ -355,6 +359,10 @@ void linux_boot( void )
     if (bi.mch_type == ATARI_MACH_AB40 && ((unsigned long)memptr & 0xff000000))
 	SERROR( "Error: Bootstrap may not allocate memory from FastRAM "
 		"on Afterburner040\n" );
+    if (getcookie("_CT2", NULL) != -1 &&
+	((unsigned long)memptr & 0xff000000))
+	SERROR( "Error: Bootstrap may not allocate memory from TT-RAM "
+		"on Centurbo2\n" );
 
     /* clearing the kernel's memory perhaps avoids "uninitialized bss"
      * types of bugs... */
@@ -565,13 +573,19 @@ static void get_cpu_infos( void )
 
 static void get_mch_type( void )
 {
-    u_long mch_cookie, ab40_cookie;
+    u_long mch_cookie;
     int rv;
 
     /* Pass contents of the _MCH cookie to the kernel */
     getcookie("_MCH", &mch_cookie);
     bi.mch_cookie = mch_cookie;
-    
+
+    bi.mch_type = ATARI_MACH_NORMAL;
+    if (getcookie("_CT2", NULL) != -1)
+	/* On the Centurbo2 board, no memory tests will give bus errors! So
+	 * don't even execute the test code below, but proceed */
+	goto finish;
+	
     __asm__ __volatile__
 	( "movel	0x8,a0\n\t"	/* save buserr vector */
 	  "movel	sp,a1\n\t" 	/* save stack pointer */
@@ -602,7 +616,6 @@ static void get_mch_type( void )
 	  : /* no inputs */
 	  : "d1", "d2", "a0", "a1", "memory" );
 
-    bi.mch_type = ATARI_MACH_NORMAL;
     switch( rv ) {
       case 1:
 	bi.mch_type = ATARI_MACH_AB40;
@@ -620,12 +633,13 @@ static void get_mch_type( void )
      * if an "AB40" cookie exists, or with 99% if it's a Falcon and has a '040
      * processor. */
     if (bi.mch_type == ATARI_MACH_NORMAL) {
-	if (getcookie("AB40", &ab40_cookie) != -1 ||
+	if (getcookie("AB40", NULL) != -1 ||
 	    ((bi.mch_cookie >> 16) == ATARI_MCH_FALCON &&
 	     bi.cputype == CPU_68040))
 	    bi.mch_type = ATARI_MACH_AB40;
     }
 
+  finish:
     printf( "Model: " );
     switch( bi.mch_cookie >> 16 ) {
       case ATARI_MCH_ST:
@@ -756,14 +770,21 @@ static void get_mem_infos( void )
 	    /* all these kinds of TT-RAM are mutually exclusive */
 
 	    /* "Original" or properly emulated TT-Ram */
-	    if (*ramtop)
+	    if (*ramtop) {
 		/* the 'ramtop' variable at 0x05a4 is not
 		 * officially documented. We use it anyway
 		 * because it is the only way to get the TTram size.
 		 * (It is zero if there is no TTram.)
 		 */
-		ADD_CHUNK( TT_RAM_BASE, *ramtop - TT_RAM_BASE,
-			   force_tt_size, "TT-RAM" );
+		if (getcookie("_CT2", NULL) != -1)
+		    /* TT-RAM starts at phys. 0x04000000 on Centurbo2 but is
+		     * remapped for TOS to 0x01000000 with the MMU */
+		    ADD_CHUNK( CT2_FAST_START, *ramtop - TT_RAM_BASE,
+			       force_tt_size, "TT-RAM" );
+		else
+		    ADD_CHUNK( TT_RAM_BASE, *ramtop - TT_RAM_BASE,
+			       force_tt_size, "TT-RAM" );
+	    }
 	    /* test for MAGNUM alternate RAM
 	     * added 26.9.1995 M. Schwingen, rincewind@discworld.oche.de
 	     */
@@ -832,7 +853,8 @@ static int getcookie(char *cookie, u_long *value)
 
     while(cookiejar[i] != 0L) {
 	if(cookiejar[i] == *(u_long *)cookie) {
-	    *value = cookiejar[i + 1];
+	    if (value)
+		*value = cookiejar[i + 1];
 	    return 1;
 	}
 	i += 2;
@@ -1342,7 +1364,7 @@ static int get_ab040_bank_sizes( int maxres, u_long *result )
 	    if (val != addr2)
 		break;
 	} while( npages );
-	end = addr2;
+	end = npages ? addr2 : addr;
 
 	if (end - start >= MB) {
 	    if (maxres-- > 0) {
